@@ -240,10 +240,10 @@ $string['privacy:metadata:aggregatedgradinggrade'] = 'Aggregated grade for all a
 $string['privacy:metadata:assessmentgrade'] = 'Aggregated grade for the submission suggested by this assessment';
 $string['privacy:metadata:assessmentgradinggrade'] = 'Grade for providing this assessment';
 $string['privacy:metadata:assessmentgradinggradeover'] = 'Manually overridden value of the grade for providing this assessment';
-$string['privacy:metadata:assessmentid'] = 'Identifier of the assessment';
-$string['privacy:metadata:authorid'] = 'Identifier of the submission author';
+$string['privacy:metadata:assessmentid'] = 'ID of the assessment';
+$string['privacy:metadata:authorid'] = 'ID of the submission author';
 $string['privacy:metadata:dimensiongrade'] = 'Grade in the given assessment dimension';
-$string['privacy:metadata:dimensionid'] = 'Identifier of the assessment dimension';
+$string['privacy:metadata:dimensionid'] = 'ID of the assessment dimension';
 $string['privacy:metadata:example'] = 'Whether this record represents an example submission';
 $string['privacy:metadata:feedbackauthor'] = 'Feedback for the author';
 $string['privacy:metadata:feedbackauthorformat'] = 'Text format of the feedback for the author';
@@ -254,25 +254,25 @@ $string['privacy:metadata:peercomment'] = 'Comment on the given grade by the use
 $string['privacy:metadata:peercommentformat'] = 'Text format of the comment on the given grade';
 $string['privacy:metadata:preference:perpage'] = 'Number of submissions the user prefers to see on one page';
 $string['privacy:metadata:published'] = 'Whether the submission should be published to all participants once the workshop is closed';
-$string['privacy:metadata:reviewerid'] = 'Identifier of the user providing the assessment';
+$string['privacy:metadata:reviewerid'] = 'ID of the user providing the assessment';
 $string['privacy:metadata:strategy'] = 'Name of the grading strategy subplugin interpreting the record values';
 $string['privacy:metadata:submissioncontent'] = 'Content of the submission';
 $string['privacy:metadata:submissioncontentformat'] = 'Text format of the submission content';
 $string['privacy:metadata:submissiongrade'] = 'Aggregated grade for the submission written as a decimal number from interval 0..100';
 $string['privacy:metadata:submissiongradeover'] = 'Manually overridden value of the aggregated grade';
-$string['privacy:metadata:submissionid'] = 'Identifier of the submission';
+$string['privacy:metadata:submissionid'] = 'ID of the submission';
 $string['privacy:metadata:submissiontitle'] = 'Title of the submission';
 $string['privacy:metadata:subsystem:corefiles'] = 'Workshop module stores files embedded in / attached to the submission text';
 $string['privacy:metadata:subsystem:coreplagiarism'] = 'Workshop module has inbuilt support for plagiarism prevention systems';
 $string['privacy:metadata:timeaggregated'] = 'When the aggregated grade was last calculated';
 $string['privacy:metadata:timecreated'] = 'When this record was created in the database';
 $string['privacy:metadata:timemodified'] = 'When this record was last modified in the database';
-$string['privacy:metadata:userid'] = 'Identifier of the user for which aggregated grade is calculated';
+$string['privacy:metadata:userid'] = 'ID of the user for which aggregated grade is calculated';
 $string['privacy:metadata:weight'] = 'Weight of the assessment';
 $string['privacy:metadata:workshopaggregations'] = 'Holds aggregated grades for assessment';
 $string['privacy:metadata:workshopassessments'] = 'Holds information about allocated assessments of workshop module submissions';
 $string['privacy:metadata:workshopgrades'] = 'Holds information about how the assessment forms were filled with grades and comments';
-$string['privacy:metadata:workshopid'] = 'Identifier of the workshop activity';
+$string['privacy:metadata:workshopid'] = 'ID of the workshop activity';
 $string['privacy:metadata:workshopsubmissions'] = 'Holds information about workshop module submissions';
 $string['privacy:request:delete:title'] = '[Deleted]';
 $string['privacy:request:delete:content'] = 'The content has been deleted at the request of the user.';
